//---------------------------------------------------------------
// Lighting
// The first light (PIXELLIGHTS) is a Pixel light.
// Any others use vertex lights.
//
// Copyright Tall Studios Ltd, 2012.  All rights reserved.
//---------------------------------------------------------------

#define MAXLIGHTS 8
#define PIXELLIGHTS 1

// Ambient Lighting Settings
float4		colourAmbient : Ambient0;
float		fAmbientIntensity;

//Lights
const float3 HemisphericColor = float3( 0.15f, 0.2f, 0.30f);
const float3 Up = float3(0,1,0); //for the skylight
	
struct LIGHT
{
    float	iType;						// Type of light.  0=Directional, 1=Point, 2=Spotlight
    float4	vPosition: POSITION;		// Position of light (Point or Spotlight only)
    float4	vDirection;					// Direction of light (Directional or Spotlight only)
    float4	colour: COLOR;				// Colour of light
    float	fRange;						// Range of light
    float	fInnerAngle;				// Inner Angle of spotlight (Spotlight only)
    float	fOuterAngle;				// Outer Angle of spotlight (Spotlight only)
    bool	bEnabled;					// Is this light switched on?
};

uniform LIGHT	light[MAXLIGHTS];
uniform int		iNumberOfLights = 0;

uniform float SpecPow <
	string UIName = "Specular Glossiness";
	float UIMin = 0.0f;
	float UIMax = 100.0f;
	float UIStep = 0.1f;
>   = 30.0f;


//Function: Omni Lighting + Attenuation + Specular
static inline float4 OmniAttenuationSpecular(in int lightnumber, in float3 WorldPosition, float3 Normal, float3 ViewDir)
{ 
//	float3 LightDir = light[lightnumber].vPosition-WorldPosition;
//	half dist = length(LightDir);	
//	LightDir = normalize(LightDir);
//	half attenuation = 1-saturate(dist/light[lightnumber].fRange);					//Attenuation
//	attenuation = attenuation * attenuation;										//More realistic attenuation
//	float3 halfVector = normalize(LightDir + ViewDir);
//	float Specular = pow(saturate(dot(Normal,halfVector)),SpecPow)*attenuation;		//Specular term
//	float3 Lighting = saturate(dot(Normal,LightDir))*attenuation*light[lightnumber].colour;
//	return float4(Lighting,Specular);
	return float4(0,0,0,0);
}
   
//Function: Directional Lighting + Specular
static inline float4 DirectionalSpecular(int lightnumber, float3 Normal, float3 ViewDir)
{
//	float3 LightDir = normalize(light[lightnumber].vDirection);
//	float3 halfVector = normalize(LightDir + ViewDir);
//	float Specular = pow(saturate(dot(Normal,halfVector)),SpecPow);					//Specular term
//	float3 Lighting = saturate(dot(Normal,LightDir))*light[lightnumber].colour;
//	return float4(Lighting,Specular);
return float4(0,0,0,0);
}
   
//Function: Omni Lighting + Attenuation + Specular
static inline float4 SpotAttenuationSpecular(in int lightnumber, in float3 WorldPosition, float3 Normal, float3 ViewDir)
{ 
//	float3 LightDir = light[lightnumber].vPosition-WorldPosition;
//	half dist = length(LightDir);	
//	LightDir = normalize(LightDir);
//	half attenuation = 1-saturate(dist/light[lightnumber].fRange);					//Attenuation
//	attenuation = attenuation * attenuation;										//More realistic attenuation
	   
//	half coneDot = dot(normalize(light[lightnumber].vDirection),LightDir);			//Spot cone attenuation - Will be optimised without cos
//	attenuation *= saturate((coneDot-cos(light[lightnumber].fOuterAngle))/(cos(light[lightnumber].fInnerAngle)-cos(light[lightnumber].fOuterAngle)));
		
//	float3 halfVector = normalize(LightDir + ViewDir);
//	float Specular = pow(saturate(dot(Normal,halfVector)),SpecPow)*attenuation;		//Specular term
//	float3 Lighting = saturate(dot(Normal,LightDir))*attenuation*light[lightnumber].colour;
//	return float4(Lighting,Specular);
	return float4(0,0,0,0);
}

   
float4x4	matWorldViewProj : WorldViewProjection;
float4x4 matWorld : World;
float4 vCameraPosition : WORLDCAMERAPOSITION;


uniform bool b_UseNormal <
	string UIName = "Use Normal map";
> = false;
uniform bool b_UseReflection <
	string UIName = "Use Reflection map";
> = false;
uniform bool b_UseAmbient <
	string UIName = "Use ambient colour";
> = true;
uniform bool b_UseDiffuse <
	string UIName = "Use Diffuse map";
> = 1;
uniform bool b_UseSpecMask <
	string UIName = "Use Specular map";
> = 0;


texture diffusemap : DIFFUSEMAP <
	string name = "";
	string UIName = "Diffuse Texture";
	int Texcoord = 0;
	int MapChannel = 1;
	>;
	
sampler2D Diffusemap = sampler_state
{
	   Texture   = (diffusemap);
	   MinFilter = LINEAR;
	   MagFilter = LINEAR;
	   MipFilter = LINEAR;   
};

float4 DiffuseColor : COLOR <
	string UIName = "DiffuseColor";
	string UIWidget = "Color";
> = float4( 0.7,0.7,0.7, 1.0);  

	
texture normalmap : NORMALMAP <
	string name = "";
	string UIName = "Normal Texture";
	int Texcoord = 0;
	int MapChannel = 1;
	>;

// Sampler with bilinear filtering
sampler2D Normalmap = sampler_state
{
	   Texture   = (normalmap);
	   MinFilter = LINEAR;
	   MagFilter = LINEAR;
	   MipFilter = LINEAR;   
};

texture specularmap : SPECULARMAP <
	string name = "";
	string UIName = "Specular Texture";
	int Texcoord = 0;
	int MapChannel = 1;
	>;

// Sampler with bilinear filtering
sampler2D Specularmap = sampler_state
{
	   Texture   = (specularmap);
	   MinFilter = LINEAR;
	   MagFilter = LINEAR;
	   MipFilter = LINEAR;   
};

texture envcubemap : ENVCUBEMAP <
	string name = "";
	string UIName = "Cubemap Texture";
	string Type = "Cube";
	>;

// Sampler with bilinear filtering
samplerCUBE EnvCubemap = sampler_state
{
	   Texture   = (envcubemap);
	   MinFilter = LINEAR;
	   MagFilter = LINEAR;
	   MipFilter = LINEAR;   
};

float3 SpecMask : COLOR <
	string UIName = "Specular Intensity";
	string UIWidget = "Color";
> = float3( 1.0,1.0,1.0);  


uniform float fresnelPower <
	string UIName = "Fresnel power";
	float UIMin = 0.0f;
	float UIMax = 100.0f;
	float UIStep = 0.1f;
>   = 1.0f;


// -------------------------------------------------------------
// vertex shader output channels
// -------------------------------------------------------------
struct VS_OUTPUT
{
   float4 Pos: POSITION;
   float2 UV0 : TEXCOORD0;
   float3 ViewDir : TEXCOORD1;
   float3 Normal : TEXCOORD2;
   float3 Tangent : TEXCOORD3;
   float3 BiNormal : TEXCOORD4;
   float3 WorldPos : TEXCOORD5;
   float4 OtherLights : TEXCOORD6;

};



//-------------------------------------------------------------
// Vertex Shader Function (input channels)
//-------------------------------------------------------------

VS_OUTPUT VS( float4 Pos : POSITION, float3 Normal : NORMAL, float3 Tangent : TANGENT, float3 Binormal : BINORMAL, float2 UV0:TEXCOORD0 )
{
	VS_OUTPUT Out = (VS_OUTPUT) 0; 
   
	float4 WorldViewPos =   mul(Pos, matWorldViewProj); // transform Position 
	float4 WorldPos = mul(Pos, matWorld);
	float3 ViewDir = vCameraPosition - WorldPos;
	Out.ViewDir = ViewDir;
          
	Out.Pos = WorldViewPos; // transform Position
	Out.WorldPos = WorldPos;
   
	Out.Normal = mul(Normal,matWorld);
	Out.Tangent = mul(Tangent, matWorld);
	Out.BiNormal = mul(Binormal, matWorld);
   
	Out.UV0 = UV0;
	float3 WorldNormal = normalize(mul(Normal, matWorld)); 
         

	//vertex lights
	float4 TotalDiffuse	= 0;
	float3 Diffuse = 0;
	float4 VXLightDirection = 0;

	//Directional lights
	half d = 0;
	float3 LightDir;
	half dist;
	half attenuation;
	half coneDot;
	ViewDir = normalize(ViewDir);

	for (int i = PIXELLIGHTS; i < iNumberOfLights; i++)
	{    
		if (light[i].iType==0)
		{ 	
			TotalDiffuse += DirectionalSpecular(i, WorldNormal, ViewDir);	
		}
		if (light[i].iType==1)
		{ 	
			TotalDiffuse += OmniAttenuationSpecular(i, WorldPos, WorldNormal,ViewDir);
		}
		if (light[i].iType==2)
		{ 	
			TotalDiffuse += SpotAttenuationSpecular(i, WorldPos, WorldNormal,ViewDir);	
		}
	}
	Out.OtherLights = TotalDiffuse;
		  
   return Out;
}


//-------------------------------------------------------------
// Pixel Shader (input channels):output channel
//-------------------------------------------------------------
float4 PS4(float4 Pos : POSITION0, float2 UV0 : TEXCOORD0, float3 ViewDir : TEXCOORD1, float3 Normal : TEXCOORD2, float3 Tangent : TEXCOORD3, float3 BiNormal : TEXCOORD4, float3 WorldPos : TEXCOORD5, float4 OtherLights : TEXCOORD6) : COLOR
{	
	float4 LightResult = 0;
	float3 NN = Normal;
	
	ViewDir = normalize(ViewDir);
		
	Normal = float3(0,0,1);
	if (b_UseNormal) 
	{
		Normal = 2.0f*tex2D(Normalmap, UV0.xy)-1.0f;
	}

	Normal = (Normal.x * Tangent) + (Normal.y * BiNormal) + (Normal.z * NN);	//Put normal into world space
	Normal = normalize(Normal);
		
	//Fresnel term
	half fresnel = saturate(dot(ViewDir, Normal));
	fresnel = pow(1-fresnel, fresnelPower);
	
	if (b_UseSpecMask)		// Use a texture for the specular mask
	{
		SpecMask = tex2D(Specularmap, UV0.xy);
	}
	SpecMask *= fresnel;	//Multiply the two terms to save  mults

	// Adding the vertexcolors lights
	float4 Color = OtherLights; 
	float3 Specular = OtherLights.aaa;
	
	for (int i = 0; i < iNumberOfLights && i<PIXELLIGHTS; i++)
	{
		if (light[i].iType==0)
		{  
			//LightResult = DirectionalSpecular(i, Normal, ViewDir);
	float3 LightDir2 = normalize(light[i].vDirection);
	float3 halfVector2 = normalize(LightDir2 + ViewDir);
	float Specular2 = pow(saturate(dot(Normal,halfVector2)),SpecPow);					//Specular term
	float4 Lighting2 = saturate(dot(Normal,LightDir2))*light[i].colour;
//	Lighting.a = 0.5;
//	return float4(Lighting,Specular);
			Color.rgb += Lighting2.rgb;
			Specular += Lighting2.a*light[i].colour*SpecMask;
		}

		if (light[i].iType==1)
		{
			LightResult = OmniAttenuationSpecular(i, WorldPos, Normal,ViewDir);
			Color.rgb += LightResult.rgb;
			Specular += LightResult.a*light[i].colour*SpecMask;
		}

		if (light[i].iType==2)
		{ 	
			LightResult = SpotAttenuationSpecular(i, WorldPos, Normal,ViewDir);
			Color.rgb += LightResult.rgb;
			Specular += LightResult.a*light[i].colour*SpecMask;	
		}
	}


	if (b_UseAmbient)
	{
		half LerpFactor = (dot(Normal, Up) + 1) / 2;  
		Color.rgb += HemisphericColor*LerpFactor;
   	}

	Color.rgb += colourAmbient * fAmbientIntensity;
 
	if (b_UseDiffuse)
	{
		DiffuseColor = tex2D(Diffusemap, UV0.xy);
	}
 
	Color.rgb *= DiffuseColor.rgb;
	Color.rgb += Specular;

	if (b_UseReflection) 
	{
		float3 Reflections = texCUBE(EnvCubemap,-reflect(normalize(ViewDir),Normal))*SpecMask;
		Color.rgb += Reflections;
	}

	return float4(Color.rgb,1);
}


//-------------------------------------------------------------
// Name technique
//-------------------------------------------------------------
technique MaterialShader
{
	pass NoBones
	{
		// compiler directives
		VertexShader = compile vs_2_0 VS();
		ZEnable = true; 
		CullMode = CCW; 
		ZWriteEnable = true; 
		AlphaBlendEnable = false;
		PixelShader  = compile ps_2_0 PS4();
	}
}